#!/usr/bin/env python

# Written by Bram Cohen
# updated by TorrentFlux.com

from BitTorrent.download import download
from threading import Event
from os.path import abspath
from sys import argv, stdout
from cStringIO import StringIO
from time import time

def hours(n):
    if n == -1:
        return '<unknown>'
    if n == 0:
        return 'complete!'
    n = long(n)
    h, r = divmod(n, 60 * 60)
    m, sec = divmod(r, 60)
    if h > 1000000:
        return '<unknown>'
    if h > 0:
        return '%d hour %02d min %02d sec' % (h, m, sec)
    else:
        return '%d min %02d sec' % (m, sec)

class HeadlessDisplayer:
    def __init__(self):
        self.done = False
        self.file = ''
        self.percentDone = ''
        self.timeEst = ''
        self.downloadTo = ''
        self.downRate = ''
        self.upRate = ''
        self.downTotal = 0
        self.upTotal = 0
        self.errors = []
        self.last_update_time = 0
        self.statFile = 'percent.txt'
        self.autoShutdown = 'False'
        self.user = 'unknown'
        self.size = 0
        self.shareKill = '100'
        self.percentShare = ''
        self.stoppedAt = ''

    def finished(self):
        self.done = True
        self.percentDone = '100'
        self.timeEst = 'Download Succeeded!'
        self.downRate = ''
        self.display({})
        if self.autoShutdown == 'True':
            self.upRate = ''
            if self.stoppedAt == '':
                self.writeStatus()
            raise KeyboardInterrupt

    def failed(self):
        self.done = True
        self.percentDone = '0'
        self.timeEst = 'Download Failed!'
        self.downRate = ''
        self.display({})
        if self.autoShutdown == 'True':
            self.upRate = ''
            if self.stoppedAt == '':
                self.writeStatus()
            raise KeyboardInterrupt

    def error(self, errormsg):
        self.errors.append(errormsg)
        self.display({})

    def display(self, dict):
        if self.last_update_time + 0.1 > time() and dict.get('fractionDone') not in (0.0, 1.0) and not dict.has_key('activity'):
            return
        self.last_update_time = time()
        if dict.has_key('spew'):
            print_spew(dict['spew'])
        if dict.has_key('fractionDone'):
            self.percentDone = str(float(int(dict['fractionDone'] * 1000)) / 10)
        if dict.has_key('timeEst'):
            self.timeEst = hours(dict['timeEst'])
        if dict.has_key('activity') and not self.done:
            self.timeEst = dict['activity']
        if dict.has_key('downRate'):
            self.downRate = '%.2f kB/s' % (float(dict['downRate']) / (1 << 10))
        if dict.has_key('upRate'):
            self.upRate = '%.2f kB/s' % (float(dict['upRate']) / (1 << 10))
        if dict.has_key('upTotal'):
            self.upTotal = dict['upTotal']*1024*1024
        if dict.has_key('downTotal'):
            self.downTotal = dict['downTotal']*1024*1024
        for err in self.errors:
            print 'ERROR:\n' + err + '\n'
        if self.stoppedAt == '':
            self.writeStatus()

    def chooseFile(self, default, size, saveas, dir):
        self.file = '%s (%.1f MB)' % (default, float(size) / (1 << 20))
        self.size = size
        if saveas != '':
            default = saveas
        self.downloadTo = abspath(default)
        return default

    def writeStatus(self):
        downRate = self.downTotal
        die = False
        f=open(self.statFile,'r')
        running = f.read(1)
        f.close
        if running == '0':
            if self.stoppedAt == '':
                if self.percentDone == '100':
                    self.stoppedAt = '100'
                else:
                    self.stoppedAt = str((float(self.percentDone)+100)*-1)
                    self.timeEst = 'Torrent Stopped'
            die = True
            self.upRate = ''
            self.downRate = ''
            self.percentDone = self.stoppedAt
        else:
            if downRate == 0 and self.upTotal > 0:
                downRate = (float(self.size) / (1 << 20))
            if self.done:
                self.percentDone = '100'
                downRate = (float(self.size) / (1 << 20))
            if self.upTotal > 0:
                self.percentShare = '%.1f' % ((float(self.upTotal)/float(downRate))*100)
            else:
                self.percentShare = '0.0'
            if self.done and self.percentShare is not '' and self.autoShutdown == 'False':
                if (float(self.percentShare) >= float(self.shareKill)) and (self.shareKill != '0'):
                    die = True
                    self.upRate = ''
        f=open(self.statFile, 'w')
        f.write(running + '\n')
        f.write(self.percentDone + '\n')
        f.write(self.timeEst + '\n')
        f.write(self.downRate + '\n')
        f.write(self.upRate + '\n')
        f.write(self.user + '\n')
        f.write('n/a' + '\n')
        f.write('n/a' + '\n')
        f.write(self.percentShare + '\n')
        f.write(self.shareKill + '\n')
        f.write(str(self.upTotal) + '\n')
        f.write(str(self.downTotal) + '\n')
        f.write(str(self.size))
        for err in self.errors:
            f.write('\n' + err)
        f.close
        if die:
            raise KeyboardInterrupt

    def newpath(self, path):
        self.downloadTo = path

def print_spew(spew):
    s = StringIO()
    s.write('\n\n\n')
    for c in spew:
        s.write('%20s ' % c['ip'])
        if c['initiation'] == 'local':
            s.write('l')
        else:
            s.write('r')
        rate, interested, choked = c['upload']
        s.write(' %10s ' % str(int(rate)))
        if c['is_optimistic_unchoke']:
            s.write('*')
        else:
            s.write(' ')
        if interested:
            s.write('i')
        else:
            s.write(' ')
        if choked:
            s.write('c')
        else:
            s.write(' ')

        rate, interested, choked, snubbed = c['download']
        s.write(' %10s ' % str(int(rate)))
        if interested:
            s.write('i')
        else:
            s.write(' ')
        if choked:
            s.write('c')
        else:
            s.write(' ')
        if snubbed:
            s.write('s')
        else:
            s.write(' ')
        s.write('\n')
    print s.getvalue()

def run(autoDie,shareKill,statusFile,userName,params):

    h = HeadlessDisplayer()
    h.statFile = statusFile
    h.autoShutdown = autoDie
    h.shareKill = shareKill
    h.user = userName
    
    download(params, h.chooseFile, h.display, h.finished, h.error, Event(), h.newpath)
    if not h.done:
        h.failed()

if __name__ == '__main__':
    run(argv[1],argv[2],argv[3],argv[4],argv[5:])
